@@ -16,15 +16,36 @@ misc.py
 ####################
 Code facilitating random aspects of this package.
 """
-
+import os
 import sys
 import inspect
 import smtplib
 import base64
 import time
+import re
 from email.MIMEMultipart import MIMEMultipart
 from email.MIMEText import MIMEText
 
+from collections import defaultdict
+
+
+def get_version_number(path_to_setup):
+    """
+    Provides access to current version info contained in setup.py
+    """
+    
+    setup_path = path_to_setup
+    with open(setup_path, 'rb') as f:
+        match = re.search(
+            '\s*[\'"]?version[\'"]?\s*[=:]\s*[\'"]?([^\'",]+)[\'"]?',
+            f.read().decode('utf-8'), re.I)
+
+    if match:
+        version_string = match.group(1)
+        return version_string
+
+    else:
+        print("No version definition found in ", setup_path)
 
 
 class Bunch(dict):
@@ -67,38 +88,39 @@ def email_notification(sender,to,subject,txt,pw,server_info):
     
     .. todo:: **DONE** make ``email_notification()`` adjustable for other email servers
     """
+    if sender:
+        msg = MIMEMultipart()
+        msg['From'] = sender
+        msg['To'] = to
+        msg['Subject'] = subject
+        msg.attach(MIMEText(txt))
     
-    msg = MIMEMultipart()
-    msg['From'] = sender
-    msg['To'] = to
-    msg['Subject'] = subject
-    msg.attach(MIMEText(txt))
-
-    host = server_info['host']
-    port = server_info['port']
-    server = smtplib.SMTP(host, port)
+        host = server_info['host']
+        port = server_info['port']
+        server = smtplib.SMTP(host, port)
+        
+        try:
+            server.ehlo()
+            server.starttls()
+            server.ehlo()
+            server.login(sender,pw)
+            server.sendmail(sender,to,msg.as_string())
+            server.close()
+        except (smtplib.SMTPAuthenticationError,
+                smtplib.SMTPConnectError,
+                smtplib.SMTPDataError,
+                smtplib.SMTPException,
+                smtplib.SMTPHeloError,
+                smtplib.SMTPRecipientsRefused,
+                smtplib.SMTPResponseException,
+                smtplib.SMTPSenderRefused,
+                smtplib.SMTPServerDisconnected) as e:
+            
+            sys.stderr.write("Warning: %s was caught while trying to send your mail.\nSubject:%s\n" % (e.__class__.__name__,subject))
     
-    try:
-        server.ehlo()
-        server.starttls()
-        server.ehlo()
-        server.login(sender,pw)
-        server.sendmail(sender,to,msg.as_string())
-        server.close()
-    except (smtplib.SMTPAuthenticationError,
-            smtplib.SMTPConnectError,
-            smtplib.SMTPDataError,
-            smtplib.SMTPException,
-            smtplib.SMTPHeloError,
-            smtplib.SMTPRecipientsRefused,
-            smtplib.SMTPResponseException,
-            smtplib.SMTPSenderRefused,
-            smtplib.SMTPServerDisconnected) as e:
-        
-        sys.stderr.write("Warning: %s was caught while trying to send your mail.\nSubject:%s\n" % (e.__class__.__name__,subject))
-
-        server.rset()
-        
+            server.rset()
+    else:
+        pass
         
     
 def get_time():
@@ -110,4 +132,20 @@ def get_time():
         
     
 
-    
\ No newline at end of file
+def map_condition_groups(yargs):
+    """
+    creates a Bunch obj ``groups`` with key='experiment_id' from ``yargs``, value=list(condition_queue objects with 'experiment_id')
+    
+    :param yargs: argument object generated from the yaml config file
+    :returns: ``groups``
+    """
+    groups = defaultdict(list)
+    for condition in yargs.condition_queue:
+        groups[condition['experiment_id']].append(condition)
+    groups = Bunch(dict(groups))
+    return groups
+
+def uniques(seq):
+    seen = set()
+    seen_add = seen.add
+    return [ x for x in seq if x not in seen and not seen_add(x)]
\ No newline at end of file
